import ConfigParser
import getpass
import os
import sys

import requests
from termcolor import colored

from api import API, EnvDoesNotExist, get_token, NoDeploymentVars, ProjectDoesNotExist


BASE_HTTP_ERROR_MESSAGE = 'An error occurred connecting to the server. Is your URL correct?'
CONFIG_SECTION = 'hoard'
GLOBAL = os.path.expanduser('~/.hoardrc')
LOCAL = os.path.join(os.getcwd(), '.hoard')


class FrontEnd(object):
    def config_get(self, key):
        config = ConfigParser.ConfigParser()
        config.read((GLOBAL, LOCAL))  # local will override global

        try:
            value = config.get(CONFIG_SECTION, key)
        except ConfigParser.NoOptionError:
            value = False

        return value

    def config_set(self, key, data):
        config = ConfigParser.ConfigParser()
        config.read(GLOBAL)
        if not config.has_section(CONFIG_SECTION):
            config.add_section(CONFIG_SECTION)
        config.set(CONFIG_SECTION, key, data)
        with open(GLOBAL, 'wb') as config_file:
            config.write(config_file)

    def env(self, args):
        self.preflight()
        if args.env and not args.all:
            env = self.config_get('env')
            if env:
                print(env)
            else:
                print('No env set')

        if args.all:
            try:
                for env in self.api.envs():
                    print(env['name'])
            except requests.HTTPError:
                self.exit_with_error(BASE_HTTP_ERROR_MESSAGE)

    def env_does_not_exist(self, project, env):
        print('"{0}" isn\'t an environment for "{1}", try:'.format(env, project))
        for env in self.api.projects(project)['envs']:
            print('  * {0}'.format(env))
        sys.exit(1)

    def exit_with_error(self, msg, args=()):
        sys.stderr.write(colored(msg.format(*args) + '\n', 'red'))
        sys.exit(1)

    def get(self, args):
        self.preflight()
        project = self.get_var('project', args.project)
        env = self.get_var('env', args.env)

        try:
            pairs = self.api.pairs(project, env)
        except requests.HTTPError:
            self.exit_with_error(BASE_HTTP_ERROR_MESSAGE)
        except NoDeploymentVars:
            msg = '"{0}: {1}" has no environment variables.'
            print(msg.format(project, env))
        except ProjectDoesNotExist:
            self.project_does_not_exist(project)
        except EnvDoesNotExist:
            self.env_does_not_exist(project, env)
        else:
            for pair in pairs.items():
                print('{0}={1}'.format(*pair))

    def get_var(self, var, cli_arg):
        v = cli_arg if cli_arg else self.config_get(var)
        if not v:
            self.exit_with_error('No {0} specified. Use --{0} or set it in your local .hoard'.format(var))
        return v

    def login(self, args=None):
        url = self.config_get('url')
        if not url:
            self.set_url()

        print('hoard needs to request an access token.')
        print('Your username and password never stored. (Check ~/.hoardrc).\n')
        username = raw_input('Username: ')
        password = getpass.getpass()

        try:
            token = get_token(url, username, password)
        except requests.HTTPError:
            self.exit_with_error(BASE_HTTP_ERROR_MESSAGE)

        self.config_set('token', token)
        self.api = API(url, token)
        print('You have successfully logged in')

    def logout(self, args):
        config = ConfigParser.ConfigParser()
        config.read(GLOBAL)
        config.remove_option(CONFIG_SECTION, 'token')
        with open(GLOBAL, 'wb') as config_file:
            config.write(config_file)

    def preflight(self):
        if not os.path.exists(GLOBAL):
            self.set_url()
            self.login()

        if os.path.exists(GLOBAL) and self.config_get('url') and self.config_get('token'):
            self.api = API(self.config_get('url'), self.config_get('token'))
            return

        # url is missing
        if not self.config_get('url'):
            self.set_url()

        if not self.config_get('token'):
            self.login()

    def project(self, args):
        self.preflight()
        if args.project and not args.list and not args.envs:
            project = self.config_get('project')
            if not project:
                print('No project set')
                sys.exit()

            print('Deployed with:')
            try:
                for env in self.api.projects(project)['envs']:
                    print('  * {0}'.format(env))
            except requests.HTTPError:
                self.exit_with_error(BASE_HTTP_ERROR_MESSAGE)

        if args.list and not args.envs:
            try:
                for project in self.api.projects():
                    print(project['name'])
            except requests.HTTPError:
                self.exit_with_error(BASE_HTTP_ERROR_MESSAGE)

        if args.envs and not args.list:
            try:
                for env in self.api.projects(args.project)['envs']:
                    print(env)
            except requests.HTTPError:
                self.exit_with_error(BASE_HTTP_ERROR_MESSAGE)

    def project_does_not_exist(self, project):
        msg = '"{0}" doesn\'t exist. To list projects use `hoard project --list`'
        self.exit_with_error(msg.format(project))

    def rm(self, args):
        self.preflight()

        project = self.get_var('project', args.project)
        env = self.get_var('env', args.env)
        try:
            for key in args.rm:
                self.api.rm(project, env, key)
        except ProjectDoesNotExist:
            self.project_does_not_exist(project)
        except EnvDoesNotExist:
            self.env_does_not_exist(project, env)

    def set(self, args):
        self.preflight()
        project = args.project if args.project else self.config_get('project')
        env = args.env if args.env else self.config_get('env')

        pairs = dict({tuple(pair.split('=', 1)) for pair in args.set})

        try:
            self.api.pairs(project, env, pairs)
        except requests.HTTPError:
            self.exit_with_error(BASE_HTTP_ERROR_MESSAGE)
        except ProjectDoesNotExist:
            self.project_does_not_exist(project)
        except EnvDoesNotExist:
            self.env_does_not_exist(project, env)
        else:
            print('Vars set for "{0}: {1}"'.format(project, env))
            for pair in args.set:
                print(pair)

    def set_url(self):
        print("~/.hoardrc doesn't have a URL. Generating now.")
        url = raw_input('hoard URL: ')
        if not url.startswith(('http://', 'https://')):
            url = 'http://{0}'.format(url)
        self.config_set('url', url)



import json
import os

import requests


class NoDeploymentVars(Exception): pass
class EnvDoesNotExist(Exception): pass
class ProjectDoesNotExist(Exception): pass


def get_token(url, username, password):
    payload = {'username': username, 'password': password}
    r = requests.post(os.path.join(url, 'login'), data=payload)
    r.raise_for_status()
    return r.json()['token']


class API(object):
    def __init__(self, url, token):
        self.headers = {
            'Authorization': 'Token {0}'.format(token),
            'Content-Type': 'application/json',
        }
        self.url = url

    def _delete(self, url):
        return requests.delete(os.path.join(self.url, url), headers=self.headers)

    def _get(self, url):
        return requests.get(os.path.join(self.url, url), headers=self.headers)

    def _list_or_detail(self, url, name):
        if name:
            url = os.path.join(url, name)

        r = self._get(url)
        r.raise_for_status()
        return r.json()

    def _put(self, url, payload):
        return requests.put(os.path.join(self.url, url), headers=self.headers, data=json.dumps(payload))

    def envs(self, name=None):
        url = 'envs'
        return self._list_or_detail(url, name)

    def pairs(self, project, env, payload=None):
        url = os.path.join('projects', project, 'envs', env)

        if payload:
            r = self._put(url, payload)
        else:
            r = self._get(url)

            if r.ok:
                if not r.json():
                    raise NoDeploymentVars
                return r.json()

        if r.status_code == requests.codes.not_found:
            try:
                projects = self.projects(project)
            except requests.exceptions.HTTPError:
                raise ProjectDoesNotExist
            else:
                if not env in projects['envs']:
                    raise EnvDoesNotExist

        r.raise_for_status()

    def projects(self, name=None):
        url = 'projects'
        return self._list_or_detail(url, name)

    def rm(self, project, env, key):
        url = os.path.join('projects', project, 'envs', env, 'keys', key)
        r = self._delete(url)

        if r.status_code == requests.codes.not_found:
            try:
                projects = self.projects(project)
            except requests.exceptions.HTTPError:
                raise ProjectDoesNotExist
            else:
                if not env in projects['envs']:
                    raise EnvDoesNotExist

        r.raise_for_status()




import argparse

from frontend import FrontEnd


def run():
    f = FrontEnd()

    parser = argparse.ArgumentParser(
        prog='hoard',
        epilog="See '%(prog)s <command> --help' for more help on a specific command."
    )

    sub_parsers = parser.add_subparsers(title='Commands')

    get_parser = sub_parsers.add_parser('get', help='Retrieve variables for a specific deployment')
    get_parser.add_argument('get', action='store_true', help=argparse.SUPPRESS)
    get_parser.add_argument('-p', '--project')
    get_parser.add_argument('-e', '--env')
    get_parser.set_defaults(func=f.get)

    set_parser = sub_parsers.add_parser('set', help='Create/update variable(s) in a specific deployment.')
    set_parser.add_argument('set', nargs='+', metavar='VAR=value')
    set_parser.add_argument('-p', '--project')
    set_parser.add_argument('-e', '--env')
    set_parser.set_defaults(func=f.set)

    rm_parser = sub_parsers.add_parser('rm', help='Delete variable(s) in a specific deployment.')
    rm_parser.add_argument('rm', nargs='+', metavar='VAR')
    rm_parser.add_argument('-p', '--project')
    rm_parser.add_argument('-e', '--env')
    rm_parser.set_defaults(func=f.rm)

    project_parser = sub_parsers.add_parser('project', help='Project specific commands.')
    project_parser.add_argument('project', nargs='?', metavar='PROJECT', default='', help='default to project specified in ~/.hoardrc or .hoard')
    project_parser.add_argument('--add', metavar='PROJECT')
    project_parser.add_argument('-e', '--envs', action='store_true', help='show specified project\'s envs')
    project_parser.add_argument('-l', '--list', action='store_true', help='list all projects')
    project_parser.set_defaults(func=f.project)

    env_parser = sub_parsers.add_parser('env', help='Environment specific commands.')
    env_parser.add_argument('env', action='store_true', help=argparse.SUPPRESS)
    env_parser.add_argument('-a', '--all', action='store_true', help='show all envs')
    env_parser.set_defaults(func=f.env)

    login_parser = sub_parsers.add_parser('login', help='Authenticate with the server backend.')
    login_parser.add_argument('login', action='store_true', help=argparse.SUPPRESS)
    login_parser.set_defaults(func=f.login)

    logout_parser = sub_parsers.add_parser('logout', help='Clear auth details.')
    logout_parser.add_argument('logout', action='store_true', help=argparse.SUPPRESS)
    logout_parser.set_defaults(func=f.logout)

    args = parser.parse_args()
    args.func(args)

if __name__ == '__main__':
    run()




from setuptools import setup


install_requires = ['requests==1.1.0', 'termcolor==1.1.0']


setup(
    name='hoard',
    version='1.1.1',
    description='',
    long_description=open('README.rst').read() + '\n\n' +
                     open('CHANGELOG.rst').read(),
    author='George Hickman',
    author_email='george@ghickman.co.uk',
    url='http://github.com/ghickman/hoard-cli',
    license=open('LICENSE').read(),
    packages=['hoard'],
    entry_points={'console_scripts': ['hoard = hoard.main:run']},
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: MacOS',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
    ),
    install_requires=install_requires,
)

